;///////////////////////////////////////////////////////////////////////////////////////////////////
;/					    			Musical Key Reference 2.0									   /
;/								Written by Donald Straney 3/30/03								   /
;/							Designed for the Z80/TI-86 graphing calculator		   				   /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////DEFINITIONS/////////////////////////////////////////////////

#include "ti86asm.inc"				;Required include files
#include "ti86ops.inc"

;CONSTANTS

datacol 	equ		65				;Small cursor position for displaying data for each key
datarow 	equ		55

helpcol		equ		70				;Small cursor position for displaying keypress help data
helprow		equ		10

;VARS

AddrUp		equ 	$ca00			;Pointers to next and previous key routines
AddrDown	equ		$ca02

.org _asm_exec_ram					;Start at asm_exec_ram

;///////////////////////////////////////////START OF PROGRAM////////////////////////////////////////

  nop								;Display shell title
  jp Major

.dw 0,ShellTitle

ShellTitle:
.db "Musical Key Reference 2.0",0

Major:								;Display major keys
									;Don't need to clear screen, top part is overwritten when bar is drawn,
									;bottom part is cleared by call to ScreenClear in DrawMajor routine
  call DrawBar						;Draw title bar					
  call DrawMajor					;Draw major keys in circle of fifths
  call SelectCharCMN				;Select C Major as first key
  jr KeyLoop						;Check for keypresses

Minor:
									;Don't need to clear screen, top part is overwritten when bar is drawn,
									;bottom part is cleared by call to ScreenClear in DrawMinor routine
  call DrawBar						;Draw title bar
  call DrawMinor					;Draw minor keys in circle of fifths
  call SelectCharAmN				;Select a minor as starting key

KeyLoop:							;Keypress handler
  call _getkey						;Go into APD until key is pressed
  cp kUp							;Is up key?
  call z,GoToUp						;If so, call routine to select previous char
  cp kDown							;Is down key?
  call z,GoToDown					;If so, call routine to select next char
  cp kVarx							;Is x-Var key?
  jr z,GoToOtherKey					;If so, switch between major/minor
  cp kNext							;Is more key?
  jr z,ShowTheory					;If so, show theory info
  cp kExit							;Is exit key?
  jp z,_clrScrn						;If so, clear screen (for users without shell) and exit
  jr KeyLoop						;If none of the above, loop until valid key is pressed  

GoToOtherKey:						;Switch keys
  ld a,(KeyStatus)					;Get key status
  or a								;Is 0? (same as cp 0)
  jr z,Minor						;If so, switch to minor
  jr Major							;Otherwise, switch to major

DrawMinor:							;Draw minor circle of fifths
  ld a,1							;Show that is in minor mode
  call DrawInit						;Initialize LCD
  ld hl,MinorCircle1				;Display circle of fifths
  call Draw_Newline
  ld hl,MinorCircle2
  call Draw_Newline
  ld hl,MinorCircle3
  call Draw_Newline
  ld hl,MinorCircle4
  call Draw_Newline
  ld hl,MinorCircle5
  call Draw_Newline
  ld hl,MinorCircle6
  call Draw_Newline
  ld hl,MinorCircle7
  jp _puts

DrawMajor:							;Draw major circle of fifths
  xor a								;Show that is in major mode
  call DrawInit						;Initialize LCD for draw
  ld hl,MajorCircle1				;Display circle of fifths
  call Draw_Newline
  ld hl,MajorCircle2
  call Draw_Newline
  ld hl,MajorCircle3
  call Draw_Newline
  ld hl,MajorCircle4
  call Draw_Newline
  ld hl,MajorCircle5
  call Draw_Newline
  ld hl,MajorCircle6
  call Draw_Newline
  ld hl,MajorCircle7
  jp _puts

ShowTheory:
  ld hl,TheoryName					;Find theory reference code string
  call _MOV10TOOP1
  call _FINDSYM
  jp c,Major						;If not found, go back to major keys
  call _exec_assembly				;If found, run it
  jp Major							;then return to major keys

;///////////////////////////////////////////ROUTINES////////////////////////////////////////////////

Draw_Newline:						;Draw text at (hl), then go to new line
  call _puts
  jp _newline

DrawInit:							;Screen init routines common to both draw routines
  ld (KeyStatus),a					;Set key status
  call ScreenClear					;Clear below 8th line of screen
  call DrawHelp						;Draw key help message
  call _homeup						;Position cursor
  jp _newline

ScreenClear:
  ld hl,$fc80						;Start screen clear about 8 lines down
  ld bc,$ffff						;End of screen address
ClearLoop:
  ld (hl),0							;Clear selected byte of screen
  push hl							;Save screen pointer
  sbc hl,bc							;Is at end of screen
  pop hl
  ret z								;If it is, end loop and routine
  inc hl							;Otherwise, point to next byte on screen
  jr ClearLoop						;and go again

DrawBar:							;Draws title bar
  ld hl,Bardata						;Set copy from pointer to title bar data
  ld de,$fc00						;Set copy to pointer to beginning of screen
  ld bc,112							;Set to copy 112 bytes-7 rows
  ldir								;Copy
  set textInverse,(iy+textflags)	;Set for inverse text draw for title bar text
  xor a
  ld (_penRow),a					;Set to start text on 1st row
  ld a,25							;SET STARTING COLUMN FOR TITLE BAR TEXT
  ld (_penCol),a					;Set to start text on column defined by user
  ld hl,Title						;Point to title
  call _vputs						;Draw title bar text
  ld a,(KeyStatus)					;Look at major/minor setting
  or a
  call z,DrawMinorStr				;If minor, draw minor on title bar 
  call nz,DrawMajorStr				;If major, draw major on title bar
  res textInverse,(iy+textflags)	;Set back to normal text draw
  ret

DrawHelp:							;Display key help message
  ld a,helpcol						;Position cursor
  ld (_penCol),a
  ld a,helprow
  ld (_penRow),a
  ld hl,KeyHelp1					;Draw key help message, line 1
  call _vputs
  ld a,helpcol						;Move cursor to next line
  ld (_penCol),a
  ld a,helprow+6
  ld (_penRow),a
  ld hl,KeyHelp2					;Draw key help message, line 2
  call _vputs
  ld a,helpcol						;Move cursor to next line
  ld (_penCol),a
  ld a,helprow+12
  ld (_penRow),a
  ld hl,KeyHelp3					;Draw key help message, line 3
  call _vputs
  ld a,helprow+18					;Move cursor to next line
  ld (_penRow),a
  ld a,helpcol
  ld (_penCol),a
  ld a,(KeyStatus)					;Get key status
  or a
  jr z,DrawMinorStr					;If in major key mode, display 'switch to minor' message
DrawMajorStr:						;Otherwise in minor mode, display 'switch to major' message
  ld hl,MajorStr					;Set to draw "major"
  jr DrawStr
DrawMinorStr:
  ld hl,MinorStr					;Set to draw "minor"
DrawStr:
  call _vputs						;Draw
  ld a,helpcol						;Position cursor
  ld (_penCol),a
  ld a,helprow+24
  ld (_penRow),a
  ld hl,KeyHelp4					;Draw key help message, line 4
  call _vputs
  ld a,helpcol						;Position cursor
  ld (_penCol),a
  ld a,helprow+30
  ld (_penRow),a
  ld hl,KeyHelp5					;Draw key help message, line 5
  call _vputs
  xor a								;Clear a so that zero flag will be set
  ret

GoToUp:								;Jump to address of next key up
  ld hl,(AddrUp)
  jp (hl)

GoToDown:							;Jump to address of next key down
  ld hl,(AddrDown)
  jp (hl)

SelectChar:						;Routine to display box around char-b,c=x,y hl=string address
  push bc						;Save bc
  push hl						;Save hl
  ld a,(KeyStatus)				;Check on major/minor status
  or a							;Is 0? (same as cp 0)
  call z,DrawMajor				;If is 0, (major) then draw major keys
  call nz,DrawMinor				;If is 1, (minor) then draw minor keys
  pop hl						;Restore hl
  pop bc						;Restore bc
  push bc						;Save bc with starting cursor position
  ld (_curRow),bc				;Set cursor location from b,c
  ld a,$df						;Draw black box over text
  call _putc
  call _putc
  pop bc						;Now return to char starting position again,
  ld (_curRow),bc
  set textInverse,(iy+textflags);and draw inverse over black chars,
  ld a,(hl)						;the original chars again!
  call _putc
  inc hl
  ld a,(hl)
  call _putc
  res textInverse,(iy+textflags);Put back to normal text draw
  ld a,datacol					;Position cursor to display data string
  ld (_penCol),a
  ld a,datarow
  ld (_penRow),a
  dec hl						;Point back to beginning of string again
  jp _vputs						;Draw data string

;///////////////////////////////////////SELECT CHARACTER ROUTINES///////////////////////////////////

SelectCharCMN:						;Set up registers for calling SelectChar (in this case, for C Major)
;  ld b,5							;Set char location on screen
;  ld c,1
  ld bc,$0501
  ld hl,CMN							;Point to data string
  call SelectChar					;Draw selection gfx-this is done first because hl is needed after this for previous key pointer
  ld hl,SelectCharFMN				;Point to routine to select previous key (F Major)
  ld de,SelectCharGMN				;Point to routine to select next key (G Major)
  jr SelectCharDMN-2				;Set as pointers used by keypress handler

SelectCharGMN:
;  ld b,8
;  ld c,2
  ld bc,$0702
  ld hl,GMN
  call SelectChar
  ld hl,SelectCharCMN
  ld de,SelectCharDMN
  jr SelectCharAMN-2

SelectCharDMN:
;  ld b,10
;  ld c,3
  ld bc,$0903
  ld hl,DMN
  call SelectChar
  ld hl,SelectCharGMN
  ld de,SelectCharAMN
  jr SelectCharEMN-2

SelectCharAMN:
;  ld b,10
;  ld c,4
  ld bc,$0904
  ld hl,AMN
  call SelectChar
  ld hl,SelectCharDMN
  ld de,SelectCharEMN
  jr SelectCharBMN-2

SelectCharEMN:
;  ld b,10
;  ld c,5
  ld bc,$0905
  ld hl,EMN
  call SelectChar
  ld hl,SelectCharAMN
  ld de,SelectCharBMN
  jr SelectCharFMS-2

SelectCharBMN:
;  ld b,9
;  ld c,6
  ld bc,$0806
  ld hl,BMN
  call SelectChar
  ld hl,SelectCharEMN
  ld de,SelectCharFMS
  jr SelectCharGMF-2

SelectCharFMS:
;  ld b,7
;  ld c,7
  ld bc,$0607
  ld hl,FMS
  call SelectChar
  ld hl,SelectCharBMN
  ld de,SelectCharGMF
  jr SelectCharDMF-2

SelectCharGMF:
;  ld b,4
;  ld c,7
  ld bc,$0307
  ld hl,GMF
  call SelectChar
  ld hl,SelectCharFMS
  ld de,SelectCharDMF
  jr SelectCharAMF-2

SelectCharDMF:
;  ld b,3
;  ld c,6
  ld bc,$0106
  ld hl,DMF
  call SelectChar
  ld hl,SelectCharGMF
  ld de,SelectCharAMF
  jr SelectCharEMF-2

SelectCharAMF:
;  ld b,1
;  ld c,5
  ld bc,$0005
  ld hl,AMF
  call SelectChar
  ld hl,SelectCharDMF
  ld de,SelectCharEMF
  jr SelectCharBMF-2

SelectCharEMF:
;  ld b,1
;  ld c,4
  ld bc,$0004
  ld hl,EMF
  call SelectChar
  ld hl,SelectCharAMF
  ld de,SelectCharBMF
  jr SelectCharFMN-2

SelectCharBMF:
;  ld b,1
;  ld c,3
  ld bc,$0003
  ld hl,BMF
  call SelectChar
  ld hl,SelectCharEMF
  ld de,SelectCharFMN
  jr SelectCharAmN-2

SelectCharFMN:
;  ld b,3
;  ld c,2
  ld bc,$0202
  ld hl,FMN
  call SelectChar
  ld hl,SelectCharBMF
  ld de,SelectCharCMN
  jr SelectCharEmN-2



SelectCharAmN:
;  ld b,5
;  ld c,1
  ld bc,$0501
  ld hl,AmN
  call SelectChar
  ld hl,SelectCharDmN
  ld de,SelectCharEmN
  jr SelectCharBmN-2

SelectCharEmN:
;  ld b,8
;  ld c,2
  ld bc,$0702
  ld hl,EmN
  call SelectChar
  ld hl,SelectCharAmN
  ld de,SelectCharBmN
  jr SelectCharFmS-2

SelectCharBmN:
;  ld b,10
;  ld c,3
  ld bc,$0903
  ld hl,BmN
  call SelectChar
  ld hl,SelectCharEmN
  ld de,SelectCharFmS
  jr SelectCharCmS-2

SelectCharFmS:
;  ld b,10
;  ld c,4
  ld bc,$0904
  ld hl,FmS
  call SelectChar
  ld hl,SelectCharBmN
  ld de,SelectCharCmS
  jr SelectCharGmS-2

SelectCharCmS:
;  ld b,10
;  ld c,5
  ld bc,$0905
  ld hl,CmS
  call SelectChar
  ld hl,SelectCharFmS
  ld de,SelectCharGmS
  jr SelectCharDmS-2

SelectCharGmS:
;  ld b,9
;  ld c,6
  ld bc,$0806
  ld hl,GmS
  call SelectChar
  ld hl,SelectCharCmS
  ld de,SelectCharDmS
  jr SelectCharEmF-2

SelectCharDmS:
;  ld b,7
;  ld c,7
  ld bc,$0607
  ld hl,DmS
  call SelectChar
  ld hl,SelectCharGmS
  ld de,SelectCharEmF
  jr SelectCharBmF-2

SelectCharEmF:
;  ld b,4
;  ld c,7
  ld bc,$0307
  ld hl,EmF
  call SelectChar
  ld hl,SelectCharDmS
  ld de,SelectCharBmF
  jr SelectCharFmN-2

SelectCharBmF:
;  ld b,2
;  ld c,6
  ld bc,$0106
  ld hl,BmF
  call SelectChar
  ld hl,SelectCharEmF
  ld de,SelectCharFmN
  jr SelectCharCmN-2

SelectCharFmN:
;  ld b,1
;  ld c,5
  ld bc,$0005
  ld hl,FmN
  call SelectChar
  ld hl,SelectCharBmF
  ld de,SelectCharCmN
  jr SelectCharGmN-2

SelectCharCmN:
;  ld b,1
;  ld c,4
  ld bc,$0004
  ld hl,CmN
  call SelectChar
  ld hl,SelectCharFmN
  ld de,SelectCharGmN
  jr SetKeys

SelectCharGmN:
;  ld b,1
;  ld c,3
  ld bc,$0003
  ld hl,GmN
  call SelectChar
  ld hl,SelectCharCmN
  ld de,SelectCharDmN
  jr SetKeys

SelectCharDmN:
;  ld b,6
;  ld c,1
  ld bc,$0202
  ld hl,DmN
  call SelectChar
  ld hl,SelectCharGmN
  ld de,SelectCharAmN

SetKeys:						;Set hl as next key pointer, de as last key pointer
  ld (AddrUp),hl
  ex de,hl
  ld (AddrDown),hl
  ret

;///////////////////////////////////////RESOURCE STRINGS////////////////////////////////////////////

KeyStatus: .db 1					;Key status-0=Major, 1=Minor

TheoryName: .db $12,$06,"Theory"	;Name of theory prog string

MajorCircle1: .db "     C",0		;Text for major part of circle of 5ths
MajorCircle2: .db "  F    G",0		;Positions of keys: C=5,1 F=3,2 G=8,2
MajorCircle3: .db "Bb       D",0	;Bb=1,3 D=10,3
MajorCircle4: .db "Eb       A",0	;Eb=1,4 A=10,4
MajorCircle5: .db "Ab       E",0	;Ab=1,5 E=10,5
MajorCircle6: .db " Db     B",0		;Db=1,6 B=9,6
MajorCircle7: .db "   Gb F#",0		;Gb=4,7 F#=7,7

MinorCircle1: .db "     a",0		;Text for minor part of circle of 5ths
MinorCircle2: .db "  d    e",0		;Positions of keys: a=5,1 d=3,2 e=8,2
MinorCircle3: .db "g        b",0	;g=1,3 b=10,3
MinorCircle4: .db "c        f#",0	;c=1,4 f#=10,4
MinorCircle5: .db "f        c#",0	;f=1,5 c#=10,5
MinorCircle6: .db " bb     g#",0	;bb=2,6 g#=9,6
MinorCircle7: .db "   eb d#",0		;eb=4,7 d#=7,7

CMN: .db "C : No #",0				;Major data strings
GMN: .db "G : F#",0
DMN: .db "D : F,C#",0
AMN: .db "A : F,C,G#",0
EMN: .db "E : F,C,G,D#",0
BMN: .db "B : F,C,G,D,A#",0
FMS: .db "F#: F,C,G,D,A,E#",0

FMN: .db "F : Bb",0
BMF: .db "Bb: B,Eb",0
EMF: .db "Eb: B,E,Ab",0
AMF: .db "Ab: B,E,A,Db",0
DMF: .db "Db: B,E,A,D,Gb",0
GMF: .db "Gb: B,E,A,D,G,Cb",0

AmN: .db "a : No #",0			;Minor data strings
EmN: .db "e : F#",0
BmN: .db "b : F,C#",0
FmS: .db "f#: F,C,G#",0
CmS: .db "c#: F,C,G,D#",0
GmS: .db "g#: F,C,G,D,A#",0
DmS: .db "d#: F,C,G,D,A,E#",0

DmN: .db "d : Bb",0
GmN: .db "g : B,Eb",0
CmN: .db "c : B,E,Ab",0
FmN: .db "f : B,E,A,Db",0
BmF: .db "bb: B,E,A,D,Gb",0
EmF: .db "eb: F,C,G,D,A,E#",0

Title: .db "Key Reference: ",0	;Bar title

MajorStr: .db "Major",0
MinorStr: .db "Minor",0

KeyHelp1: .db $06,$07,"-select key",0	;Key usage help messages
KeyHelp2: .db "Exit-quit",0
KeyHelp3: .db "x-Var-switch to ",0
KeyHelp4: .db "More-View",0
KeyHelp5: .db "theory info",0

Bardata:
.db 00011111b						;Title bar data-7 lines
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111000b

.db 00111111b			
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111100b

.db 00111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111100b

.db 01111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 01111111b				
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 01111111b				
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 11111111b				
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b

.end
