;///////////////////////////////////////////////////////////////////////////////////////////////////
;/					    Musical Key Reference 2.0-Music Theory Subprogram						   /
;/								Written by Donald Straney 9/21/03								   /
;/							Designed for the Z80/TI-86 graphing calculator		   				   /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////DEFINITIONS/////////////////////////////////////////////////

#include "ti86asm.inc"				;Required include files

;_ldhlind	equ		4010h			; ld hl,(hl)

.org _asm_exec_ram					;Start at asm_exec_ram

;///////////////////////////////////////////START OF PROGRAM////////////////////////////////////////

  nop								;Display shell title
  jp ProgStart

.dw 0,ShellTitle

ShellTitle:
.db "Music Theory Reference",0

ProgStart:
	call DrawBar					;Draw title bar
DrawPage:
	call ScreenClear				;Clear screen except for title bar
	ld hl,$0800						;Set inital small cursor position
	ld (_penCol),hl
	ld hl,(CurrTextPos)				;Get pointer to current position in text
	call _vputs_ml					;and draw the text
KeyLoop:
	call _getkey					;Now wait for keypress
	cp kExit						;Is it the exit key?
	ret z							;If it is, return
	cp kDown						;Is it the down key?
	jr z,MoveDown
	cp kUp							;Is it the up key?
	jr z,MoveUp
	jr KeyLoop						;If none of these, loop again

MoveDown:
	ld hl,(CurrTextPos)				;Get text pointer
MoveDownLoop:
	ld a,(hl)						;Get the symbol at the current position
	or a							;Is it 0 (end-of-string char)?
	jr z,KeyLoop					;If it is, don't move down
	cp $d6							;Is it a newline char?
	jr z,MoveDownDone
	inc hl							;If not, increment pointer
	jr MoveDownLoop					;and loop again
MoveDownDone:						;If it was a newline char,
	inc hl							;point to the beginning of the next line
	ld (CurrTextPos),hl				;put that into the current text position pointer
	jr DrawPage						;and draw the page with the updated pointer

MoveUp:
	ld hl,(CurrTextPos)				;Get text pointer
	dec hl							;Decrement the pointer twice to skip the $d6 at the end of
	dec hl							;the previous line
MoveUpLoop:
	ld a,(hl)						;Get the symbol at the current position
	or a							;Is it 0 (end-of-string char)?
	jr z,KeyLoop					;If it is, don't move up
	cp $d6							;Is it a newline char?
	jr z,MoveUpDone
	dec hl							;If not, decrement pointer
	jr MoveUpLoop					;and loop again
MoveUpDone:							;If it was a newline char,
	inc hl							;point to the beginning of the next line
	ld (CurrTextPos),hl				;put that into the current text position pointer
	jr DrawPage						;and draw the page with the updated pointer

;///////////////////////////////////////////ROUTINES////////////////////////////////////////////////

ScreenClear:
  ld hl,$fc80						;Start screen clear about 8 lines down
  ld bc,$ffff						;End of screen address
ClearLoop:
  ld (hl),0							;Clear selected byte of screen
  push hl							;Save screen pointer
  sbc hl,bc							;Is at end of screen
  pop hl
  ret z								;If it is, end loop and routine
  inc hl							;Otherwise, point to next byte on screen
  jr ClearLoop						;and go again

DrawBar:							;Draws title bar
  ld hl,Bardata						;Set copy from pointer to title bar data
  ld de,$fc00						;Set copy to pointer to beginning of screen
  ld bc,112							;Set to copy 112 bytes-7 rows
  ldir								;Copy
  set textInverse,(iy+textflags)	;Set for inverse text draw for title bar text
  xor a
  ld (_penRow),a					;Set to start text on 1st row
  ld a,25							;SET STARTING COLUMN FOR TITLE BAR TEXT
  ld (_penCol),a					;Set to start text on column defined by user
  ld hl,Title						;Point to title
  call _vputs						;Draw title bar text
  res textInverse,(iy+textflags)	;Set back to normal text draw
  ret

;===============_vputs_ml by Donald Straney==============
;
;54 bytes
;Inputs: hl-Pointer to zero-terminated string, $d6 is
;		 new line
;		 _penRow-Starting row for text
;		 _penCol-Starting column for text
;
;Outputs: text drawn to screen
;

_vputs_ml:
  push hl						;Save starting position of string
_v_inc_loop:
  ld a,(hl)						;Look at current char in string
  cp $d6						;Is $d6? (new line char)
  jr z,_v_replace_zero			;If so, go to new line
  or a							;Is 0? (end-of-string char)
  jr z,_v_disp_end				;If so, end routine
  inc hl						;Point to next char
  jr _v_inc_loop				;and loop
_v_replace_zero:
  ld (hl),0						;Replace newline char with end-of-string char
  ex de,hl						;Save current position
  pop hl						;and restore beginning-of-line pointer
  push de
  call _vputs					;Draw string
  pop de
  call _v_new_line				;then go to new line
  ex de,hl						;Restore position in string at newline char
  ld (hl),$d6					;restore the $d6
  ld a,(_penRow)
  cp 64							;Is it past the end of the screen?
  ret nc						;Don't continue if it is
  inc hl						;Otherwise, point to char after $d6
  push hl						;Save that as new beginning-of-line pointer
  jr _v_inc_loop				;Go back to the loop
_v_disp_end:
  pop hl						;Remove beginning-of-line pointer from stack to prevent crash
  jp _vputs						;and draw last line

_v_new_line:					;Move small cursor down 1 row
  ld a,(_penRow)
  add a,6
  ld (_penRow),a
  xor a
  ld (_penCol),a
  ret

;///////////////////////////////////////RESOURCE STRINGS////////////////////////////////////////////

Title: .db "Music Theory Reference",0

CurrTextPos: .dw TextStart			;Pointer to current position in text

.db 0,$d6
TextStart:
.db "FORMING CHORDS WITH SCALE DEGREES:",$d6
.db "- is half step down, + is half step up",$d6,$d6
.db "major-1,3,5",$d6
.db "minor-1,3-,5",$d6
.db "augmented-1,3,5+",$d6
.db "diminished-1,3-,5-",$d6
.db "suspended fourth-1,4,5",$d6
.db "suspended second-1,2,5",$d6
.db "fifth-1,5"
.db $d6
.db "sixth-1,3,5,6",$d6
.db "seventh-1,3,5,7-",$d6
.db "major seventh-1,3,5,7",$d6
.db "minor seventh-1,3-,5,7-",$d6
.db "ninth-1,3,5,7-,9",$d6
.db "major ninth-1,3,5,7,9",$d6
.db "minor ninth-1,3-,5,7-,9",$d6
.db "eleventh-1,3,5,7-,9,11",$d6
.db "minor eleventh-1,3-,5,7-,9,11",$d6
.db "thirteenth-1,3,5,7-,9,11,13",$d6,$d6
.db "FORMING SCALES WITH WHOLE, HALF,",$d6
.db "AND 3/2 STEPS:",$d6,$d6
.db "major-W,W,H,W,W,W,H",$d6
.db "natural minor-W,H,W,W,H,W,H",$d6
.db "harmonic minor-W,H,W,W,H,",$d6
.db "W,3/2,H",$d6
.db "melodic minor-W,H,W,W,W,W,H (up)",$d6
.db "(down is natural minor)",$d6
.db "minor pentatonic-W,3/2,W,W,W,",$d6
.db "3/2,W",$d6
.db "major pentatonic-W,W,W,3/2,",$d6
.db "W,W,3/2",$d6
.db "diminished-W,H,W,H,W,H,W,H"
.db $d6,$d6
.db "ORDER OF SHARPS:",$d6,$d6
.db "F,C,G,D,A,E,B",$d6,$d6
.db "ORDER OF FLATS:",$d6,$d6
.db "B,E,A,D,G,C,F"
.db $d6,$d6
.db "MODES:",$d6
.db "ionian-(major)",$d6
.db "dorian-(minor w/6+)",$d6
.db "phrygian-(minor w/2-)",$d6
.db "lydian-(major w/4+)",$d6
.db "mixolydian-(major w/7-)",$d6
.db "aeolian-(minor)",$d6
.db "locrian-(minor w/2-,5-)",0
.db 0

Bardata:
.db 00011111b						;Title bar data-7 lines
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111000b

.db 00111111b			
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111100b

.db 00111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111100b

.db 01111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 01111111b				
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 01111111b				
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 11111111b				
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b

.end
